﻿#include-once
#include <Array.au3>
#include <Date.au3>

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;    Funktion _Feiertage($Jahr)
;
;    gibt die Feiertage des übergebenen Jahres
;    als sortiertes Array[TT.MM.JJJJ od. JJJJ/MM/TT][Feiertag][Geltungsbereich][opt. Wochentag] zurück
;   Geltungsbereich  Leerstring - kein offizieller Feiertag
;                    BG - Bundesweit Gesetzlich
;                    Regionalkennzeichen: BB-Brandenburg  BE-Berlin  BW-Baden-Württemberg  BY-Bayern
;                                         HB-Bremen  HE-Hessen  HH-Hamburg  MV-Mecklenburg-Vorpommern
;                                         NI-Niedersachsen  NW-Nordrhein-Westfalen  RP-Rheinland-Pfalz
;                                         SH-Schleswig-Holstein  SL-Saarland  SN-Sachsen
;                                         ST-Sachsen-Anhalt  TH-Thüringen
;    $DateTyp    1 - Datum als TT.MM.JJJJ (Standard)
;               0 - Datum als JJJJ/MM/TT
;   $wDay       0 - keine Rückgabe Wochentag (Standard)
;               1 - Rückgabe Wochentag ($array[n][3])
;   $sort       0 - gegliedert fix/variabel (Standard), innerhalb sortiert
;               1 - komplett sortiert
;    Autor    BugFix (bug_fix@web.de)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
Func _Feiertage($year, $DateTyp = 1, $wDay = 0, $sort = 0)
	Local $4AdvDat, $3AdvDat, $2AdvDat, $1AdvDat, $TotSoDat, $BuBDat, $MutterDat, $ErnteDat, $tmp
	Local $HDays[33][3], $a, $b, $c, $d, $e, $H1, $H2, $N, $M
	Local $Easter, $EasterDay, $EasterMonth, $EasterDate, $RestJahr, $Tempyear
	;fixe Feiertage
	$HDays[0][0] = $year & "/01/01\Neujahr\BG"
	$HDays[1][0] = $year & "/01/06\Heilige Drei Könige\BW, BY, ST"
	$HDays[2][0] = $year & "/02/14\Valentinstag\"
	$HDays[3][0] = $year & "/05/01\Maifeiertag\BG"
	$HDays[4][0] = $year & "/08/15\Mariä Himmelfahrt\BY, SL"
	$HDays[5][0] = $year & "/10/03\Tag der Deutschen Einheit\BG"
	$HDays[6][0] = $year & "/10/31\Reformationstag\BB, MV, SN, ST, TH"
	$HDays[7][0] = $year & "/11/01\Allerheiligen\BW, BY, NW, RP, SL"
	$HDays[8][0] = $year & "/12/24\Heiligabend\"
	$HDays[9][0] = $year & "/12/25\1. Weihnachtsfeiertag\BG"
	$HDays[10][0] = $year & "/12/26\2. Weihnachtsfeiertag\BG"
	$HDays[11][0] = $year & "/12/31\Silvester\"
	;variable Feiertage
	$a = Mod($year, 19)
	$b = Mod($year, 4)
	$c = Mod($year, 7)
	$H1 = Int($year / 100)
	$H2 = Int($year / 400)
	$N = 4 + $H1 - $H2
	$M = 15 + $H1 - $H2 - Floor(Int((8 * $H1 + 13) / 25))
	$d = Mod((19 * $a + $M), 30)
	$e = Mod((2 * $b + 4 * $c + 6 * $d + $N), 7)
	If $d + $e = 35 Then
		$Easter = 50
	Else
		If $d = 28 And $e = 6 And $a > 10 Then
			$Easter = 49
		Else
			$Easter = 22 + $d + $e
		EndIf
	EndIf
	If $Easter < 32 Then
		$EasterDay = $Easter
		$EasterMonth = "03"
	Else
		$EasterDay = $Easter - 31
		$EasterMonth = "04"
	EndIf
	If $EasterDay < 10 Then
		$EasterDay = "0" & $EasterDay
	EndIf
	If $year < 1900 Then ; Datumsoperationen nur mgl. wenn > 1900 , Jahr wird konvertiert
		$RestJahr = Mod($year, 100)
		If _DateIsLeapYear($year) Then
			If $RestJahr < 10 Then
				$RestJahr = "0" & $RestJahr
			EndIf
			$Tempyear = 20 & $RestJahr
		Else
			If $RestJahr < 10 Then
				$RestJahr = "0" & $RestJahr
			EndIf
			$Tempyear = 19 & $RestJahr
		EndIf
		$EasterDate = $Tempyear & "/" & $EasterMonth & "/" & $EasterDay
	Else
		$EasterDate = $year & "/" & $EasterMonth & "/" & $EasterDay
	EndIf
	Local $WFastDate = _DateAdd('d', -52, $EasterDate)
	Local $RosDat = _DateAdd('d', -48, $EasterDate)
	Local $FastDat = _DateAdd('d', -47, $EasterDate)
	Local $AschDat = _DateAdd('d', -46, $EasterDate)
	Local $GrDoDat = _DateAdd('d', -3, $EasterDate)
	Local $KarDat = _DateAdd('d', -2, $EasterDate)
	Local $OSaDat = _DateAdd('d', -1, $EasterDate)
	Local $OSoDat = $EasterDate
	Local $OMoDat = _DateAdd('d', 1, $EasterDate)
	Local $HiFaDat = _DateAdd('d', 39, $EasterDate)
	Local $PfSoDat = _DateAdd('d', 49, $EasterDate)
	Local $PfMoDat = _DateAdd('d', 50, $EasterDate)
	Local $FroDat = _DateAdd('d', 60, $EasterDate)
	; Ermitteln nicht von Ostern abhängiger, veränderlicher Feiertage
	; Muttertag = 2. Sonntag im Mai ABER wenn Pfingsten = 2.Sonntag im Mai dann ist Muttertag am 1. Sonntag
	; Der 2. Sonntag kann nur zw. dem 8. u. 14.5. liegen
	For $maitag = 8 To 14
		If _DateToDayOfWeek($year, 5, $maitag) = 1 Then
			If $maitag < 10 Then
				$maitag = "0" & $maitag
			EndIf
			$MutterDat = $year & "/05/" & $maitag
			If $MutterDat = $PfSoDat Then
				$MutterDat = _DateAdd('d', -7, $year & "/05/" & $maitag)
			EndIf
			ExitLoop
		EndIf
	Next
	; Erntedankfest 1. Sonntag im Oktober (zw. 1. u. 7.10.)
	For $oktobertag = 1 To 7
		If _DateToDayOfWeek($year, 10, $oktobertag) = 1 Then
			$oktobertag = "0" & $oktobertag
			$ErnteDat = $year & "/10/" & $oktobertag
			ExitLoop
		EndIf
	Next
	; 4.Advent = Sonntag vor 25.12. (zw. 18. u. 24.12.)
	For $deztag = 18 To 24
		If _DateToDayOfWeek($year, 12, $deztag) = 1 Then
			$4AdvDat = $year & "/12/" & $deztag
			$3AdvDat = _DateAdd('d', -7, $4AdvDat)
			$2AdvDat = _DateAdd('d', -14, $4AdvDat)
			$1AdvDat = _DateAdd('d', -21, $4AdvDat)
			$TotSoDat = _DateAdd('d', -28, $4AdvDat)
			$BuBDat = _DateAdd('d', -32, $4AdvDat)
			ExitLoop
		EndIf
	Next
	$HDays[12][0] = $WFastDate & "\Weiberfastnacht\"
	$HDays[13][0] = $RosDat & "\Rosenmontag\"
	$HDays[14][0] = $FastDat & "\Fastnacht\"
	$HDays[15][0] = $AschDat & "\Aschermittwoch\"
	$HDays[16][0] = $GrDoDat & "\Gründonnerstag\"
	$HDays[17][0] = $KarDat & "\Karfreitag\BG"
	$HDays[18][0] = $OSaDat & "\Ostersamstag\"
	$HDays[19][0] = $OSoDat & "\Ostersonntag\"
	$HDays[20][0] = $OMoDat & "\Ostermontag\BG"
	$HDays[21][0] = $HiFaDat & "\Christi Himmelfahrt\BG"
	$HDays[22][0] = $PfSoDat & "\Pfingstsonntag\"
	$HDays[23][0] = $PfMoDat & "\Pfingstmontag\BG"
	$HDays[24][0] = $MutterDat & "\Muttertag\"
	$HDays[25][0] = $FroDat & "\Fronleichnam\BW BY HE NW RP SL SN TH"
	$HDays[26][0] = $ErnteDat & "\Erntedankfest\"
	$HDays[27][0] = $BuBDat & "\Buß- und Bettag\SN"
	$HDays[28][0] = $TotSoDat & "\Totensonntag\"
	$HDays[29][0] = $1AdvDat & "\1. Advent\"
	$HDays[30][0] = $2AdvDat & "\2. Advent\"
	$HDays[31][0] = $3AdvDat & "\3. Advent\"
	$HDays[32][0] = $4AdvDat & "\4. Advent\"
	If $sort Then
		_ArraySort($HDays)
	Else
		_ArraySort($HDays, 0, 0, 11)
		_ArraySort($HDays, 0, 12)
	EndIf
	If $wDay Then ReDim $HDays[33][4]
	For $i = 0 To 32
		$tmp = StringSplit($HDays[$i][0], "\", 2)
		If $DateTyp Then ; Datum konvertieren zu TT.MM.JJJJ
			$HDays[$i][0] = StringRight($tmp[0], 2) & "." & StringMid($tmp[0], 6, 2) & "." & StringLeft($tmp[0], 4)
			If $wDay Then $HDays[$i][3] = _DateToDayOfWeek(StringLeft($tmp[0], 4), StringMid($tmp[0], 6, 2), StringRight($tmp[0], 2))
		Else
			$HDays[$i][0] = $tmp[0]
		EndIf
		$HDays[$i][1] = $tmp[1]
		$HDays[$i][2] = $tmp[2]
		If $wDay Then $HDays[$i][3] = _DateToDayOfWeek(StringLeft($tmp[0], 4), StringMid($tmp[0], 6, 2), StringRight($tmp[0], 2))
	Next
	Return $HDays
EndFunc   ;==>_Feiertage
